using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.Extensions.Caching.Memory;

namespace ReactApp.Server.Security;

internal sealed class MemoryCacheSessionStore: ITicketStore
{
    private readonly TimeSpan _expireTimeSpan;
    private static readonly MemoryCache SessionStore = new(new MemoryCacheOptions());
    private const string KeyPrefix = "AuthSessionStore-";

    public MemoryCacheSessionStore(TimeSpan expireTimeSpan)
    {
        _expireTimeSpan = expireTimeSpan;
    }
    
    public async Task<string> StoreAsync(AuthenticationTicket ticket)
    {
        var key = KeyPrefix + Guid.NewGuid();
        await RenewAsync(key, ticket);
        return key;    
    }

    public Task RenewAsync(string key, AuthenticationTicket ticket)
    {
        var options = new MemoryCacheEntryOptions();
        var expiresUtc = ticket.Properties.ExpiresUtc;
        if (expiresUtc.HasValue)
        {
            options.SetAbsoluteExpiration(expiresUtc.Value);
        }
        options.SetSlidingExpiration(_expireTimeSpan); 

        SessionStore.Set(key, ticket, options);

        return Task.CompletedTask;
    }

    public Task<AuthenticationTicket?> RetrieveAsync(string key)
    {
        SessionStore.TryGetValue(key, out AuthenticationTicket? ticket);
        return Task.FromResult(ticket);    }

    public Task RemoveAsync(string key)
    {
        SessionStore.Remove(key);
        return Task.CompletedTask;
    }
}